using System;
using System.Collections;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Drawing;
using Syncfusion.Windows.Forms.Diagram;

namespace Diagram_WF_XMLSerialization
{
    [Serializable()]
    public class myModel
    {
        [XmlIgnore]
        public FillStyle fillstyle;
        private myBackground mystyles;

        public myModel()
        {
            
        }
        
        [XmlElement("BackgroundStyle")]
        public myBackground myStyle
        {
            get
            {
                return mystyles;
            }
            set
            {
                mystyles = value;
            }
        }

        public void serial(FillStyle src)
        {
            fillstyle = src;
            mystyles = new myBackground(fillstyle);
        }
    }
    
    public class myBackground:FillStyle
    {
        protected Hashtable settings = new Hashtable(10);
        [XmlIgnore()]
        public new Color Color
        {
            get
            {
                return (Color)settings["myColor"];
            }
            set
            {
                if(!value.IsEmpty)
                settings["myColor"] = value;
            }
        }
        [XmlAttribute("Color")]
        public string XmlColor
        {
            get
            {
                return myBackground.SerializeColor(Color);
            }
            set
            {
                this.Color = myBackground.DeserializeColor(value);
            }
        }
        [XmlIgnore()]
        public new Color ForeColor
        {
            get
            {
                return (Color)settings["myForeColor"];
            }
            set
            {
                if(!value.IsEmpty)
                settings["myForeColor"] = value;
            }
        }
        [XmlAttribute("ForeColor")]
        public string XmlForeColor
        {
            get
            {
                return myBackground.SerializeColor(ForeColor);
            }
            set
            {
                this.ForeColor = myBackground.DeserializeColor(value);
            }
        }
        [XmlAttribute("FullContainerName")]
        public string FullConatinerName
        {
            get
            {
                return (string)settings["string"];
            }
            set
            {
                settings["string"] = value;
            }
        }

        public myBackground():base()
        {
            
        }

        public myBackground(FillStyle src):base(src)
        {
            Color = src.Color;
            ForeColor = src.ForeColor;
            FullConatinerName = src.FullContainerName;
        }

        protected enum ColorFormat
        {
            NamedColor,
            ARGBColor
        }

        protected static string SerializeColor(Color color)
        {
            if (color.IsNamedColor)
                return string.Format("{0}:{1}", ColorFormat.NamedColor, color.Name);
            else
                return string.Format("{0}:{1}:{2}:{3}:{4}", ColorFormat.ARGBColor, color.A, color.R, color.G, color.B);
        }

        protected static Color DeserializeColor(string color)
        {
            byte a, r, g, b;

            string[] pieces = color.Split(new char[] { ':' });

            ColorFormat colorType = (ColorFormat)Enum.Parse(typeof(ColorFormat), pieces[0], true);

            switch (colorType)
            {
                case ColorFormat.NamedColor:
                    return Color.FromName(pieces[1]);

                case ColorFormat.ARGBColor:
                    a = byte.Parse(pieces[1]);
                    r = byte.Parse(pieces[2]);
                    g = byte.Parse(pieces[3]);
                    b = byte.Parse(pieces[4]);

                    return Color.FromArgb(a, r, g, b);
            }
            return Color.Empty;
        }
    }
}
